#define F_CPU 8000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "ds18b20.h"

/*
 * ds18b20 - 
 */
uint8_t resetDS18B20() {
  uint8_t i;

  //  ,  480?s
  DS18B20_PORT &= ~(1 << DS18B20_DQ);
  DS18B20_DDR |= (1 << DS18B20_DQ); // 
  _delay_us(480);

  //  DS18B20   ,    15?s  60?s
  DS18B20_DDR &= ~(1 << DS18B20_DQ); // 
  _delay_us(60);

  //     ,     0    60?s  240?s.
  i = (DS18B20_PIN & (1 << DS18B20_DQ));
  _delay_us(420);

  //return the read value, 0=ok, 1=error
  return i;
}

/*
 *   
 */
inline void writeBitDS18B20(uint8_t bit) {
  //  0  1?s
  DS18B20_PORT &= ~(1 << DS18B20_DQ);
  DS18B20_DDR |= (1 << DS18B20_DQ); // 
  _delay_us(1);

  //    1,    (  -   )
  if (bit) {
    DS18B20_DDR &= ~(1 << DS18B20_DQ); // 
  }

  //  60?s   
  _delay_us(60);
  DS18B20_DDR &= ~(1 << DS18B20_DQ); // 
}

/*
 *   
 */
uint8_t readBitDS18B20(void) {
  uint8_t bit = 0;

  //  0  1?s
  DS18B20_PORT &= ~(1 << DS18B20_DQ);
  DS18B20_DDR |= (1 << DS18B20_DQ); // 
  _delay_us(1);

  //     14?s
  DS18B20_DDR &= ~(1 << DS18B20_DQ); // 
  _delay_us(14);

  //  
  if (DS18B20_PIN & (1 << DS18B20_DQ)) {
    bit = 1;
  }

  //  45?s   
  _delay_us(45);
  return bit;
}

/*
 *   
 */
void writeByteDS18B20(uint8_t byte) {
  uint8_t i = 8;
  while (i--) {
    writeBitDS18B20(byte & 1);
    byte >>= 1;
  }
}

/*
 *   
 */
uint8_t readByteDS18B20(void) {
  uint8_t i = 8, n = 0;
  while (i--) {
    n >>= 1;
    n |= (readBitDS18B20() << 7);
  }
  return n;
}

/*
 *  
 */
double getTempDS18B20() {
  uint8_t temperatureL;
  uint8_t temperatureH;
  double retd = 0;

  if (DS18B20_STOPINTERRUPTONREAD == 1) {
    cli();
  }

  resetDS18B20(); // 
  writeByteDS18B20(DS18B20_CMD_SKIPROM); //  (ROM)
  writeByteDS18B20(DS18B20_CMD_CONVERTTEMP); //    

  while (!readBitDS18B20()); // ,    

  resetDS18B20(); // 
  writeByteDS18B20(DS18B20_CMD_SKIPROM); //  (ROM)
  writeByteDS18B20(DS18B20_CMD_RSCRATCHPAD); //  scratchpad

  //  2   scratchpad
  temperatureL = readByteDS18B20();
  temperatureH = readByteDS18B20();

  if (DS18B20_STOPINTERRUPTONREAD == 1) {
    sei();
  }

  //   12  
  retd = ((temperatureH << 8) + temperatureL) * 0.0625;

  return retd;
}